<?php
/**
 * PrestaChamps
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file
 * If you need help please contact support@prestachamps.com
 *
 * @author    Leo Jozsa <support@prestachamps.com>
 * @copyright PrestaChamps
 * @license   commercial
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Pch_googlecachepolicy extends Module
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'pch_googlecachepolicy';
        $this->tab = 'advertising_marketing';
        $this->version = '1.0.2';
        $this->author = 'PrestaChamps';
        $this->need_instance = 0;
        
        $this->module_key = '33426fca37de63c0f3e700ced4d6089b';

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Google Cache Policy');
        $this->description = $this->l('Add Google Cache Policy');

        $this->confirmUninstall = $this->l('');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('PCH_GOOGLECACHEPOLICY_LIVE_MODE', false);

        return parent::install() &&
            $this->createBackUpFromHtAccess() &&
            // $this->registerHook('actionHtaccessCreate') &&
            $this->registerHook('displayBackOfficeHeader');
    }

    public function uninstall()
    {
        Configuration::deleteByName('PCH_GOOGLECACHEPOLICY_LIVE_MODE');

        return parent::uninstall() && $this->removeCacheFromHtAccess();
    }

    public function disable($force_all = true)
    {
        $this->removeCacheFromHtAccess();
        return true;
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        $dir = _PS_ROOT_DIR_ . '/modules/pch_googlecachepolicy/backups/';
        $files = array_diff(scandir($dir), array('.', '..'));

        $backupFiles = [];
        foreach ($files as $file) {
            if($file != 'index.php'){                

                $time = date("F d Y", filectime(_PS_ROOT_DIR_ . '/modules/pch_googlecachepolicy/backups/'.$file));
                $timeArray = [
                    'time' => $time,
                    'name' => $file,
                    'backupPath' => _MODULE_DIR_ . 'pch_googlecachepolicy/backups/'.$file
                ];
                array_push($backupFiles, $timeArray);
            }
        }

        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitPch_googlecachepolicyModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign(array(
                'module_dir'        => $this->_path,
                'moduleVersion'     => $this->version,
                'backupFiles'       => $backupFiles,
            ));

        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitPch_googlecachepolicyModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Live mode'),
                        'name' => 'PCH_GOOGLECACHEPOLICY_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Use this module in live mode'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'PCH_GOOGLECACHEPOLICY_LIVE_MODE' => Configuration::get('PCH_GOOGLECACHEPOLICY_LIVE_MODE'),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        if ((bool)Tools::getValue('PCH_GOOGLECACHEPOLICY_LIVE_MODE') == true) {
            $this->addCacheToHtAccess();
            Configuration::updateValue('PCH_GOOGLECACHEPOLICY_LIVE_MODE', true);
        } else {
            $this->removeCacheFromHtAccess();
            Configuration::updateValue('PCH_GOOGLECACHEPOLICY_LIVE_MODE', false);
        }

    }

    /**
    * Add the CSS & JavaScript files you want to be loaded in the BO.
    */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name || Tools::getValue('configure') == $this->name) {
            $this->context->controller->addCSS($this->_path.'views/css/back.css');
        }
    }

    public function createBackUpFromHtAccess()
    {
        $backUpDate = date("Y_m_d");
        $currentHtAccess = \Tools::file_get_contents(_PS_ROOT_DIR_ . '/.htaccess');

        return file_put_contents(
            _PS_ROOT_DIR_ . '/modules/pch_googlecachepolicy/backups/.htaccess_'.$backUpDate,
            PHP_EOL . $currentHtAccess
        );
    }

    public function addCacheToHtAccess()
    {
        $data = '
        ## EXPIRES CACHING FOR GOOGLE BY PRESTACHAMPS ##
        <IfModule mod_expires.c>
        ExpiresActive On
        ExpiresByType image/jpg "access 1 year"
        ExpiresByType image/jpeg "access 1 year"
        ExpiresByType image/gif "access 1 year"
        ExpiresByType image/png "access 1 year"
        ExpiresByType image/webp "access 1 year"
        ExpiresByType text/css "access 1 year"
        ExpiresByType application/pdf "access 1 year"
        ExpiresByType text/javascript "access 1 year"
        ExpiresByType text/x-javascript "access 1 month"
        ExpiresByType application/javascript "access 1 year"
        ExpiresByType application/x-javascript "access 1 month"
        ExpiresByType image/icon "access 1 month"
        ExpiresByType application/ico "access 1 month"
        ExpiresByType application/x-ico "access 1 month"
        ExpiresByType image/x-icon "access 1 year"
        ExpiresByType application/x-shockwave-flash "access 1 month"
        # Compress compressible fonts
        <IfModule mod_deflate.c>
            <IfModule mod_filter.c>
                AddOutputFilterByType DEFLATE application/x-font-ttf application/x-font-opentype image/svg+xml
            </IfModule>
        </IfModule>
        # Add a far future Expires header for fonts
        ExpiresByType application/vnd.ms-fontobject "access 1 year"
        ExpiresByType application/x-font-ttf "access 1 year"
        ExpiresByType application/x-font-opentype "access 1 year"
        ExpiresByType application/x-font-woff "access 1 year"
        ExpiresByType application/x-font-woff2 "access 1 year"
        ExpiresByType font/woff2  "access 1 year"
        ExpiresByType font/woff "access 1 year"
        ExpiresByType image/svg+xml "access 1 year"
        ExpiresByType font/otf "access 1 year"
        ExpiresByType image/vnd.microsoft.icon "access 1 year"
        </IfModule>
        #### END EXPIRES CACHING FOR GOOGLE BY PRESTACHAMPS ####';

        $currentHtAccess = \Tools::file_get_contents(_PS_ROOT_DIR_ . '/.htaccess');

        $validateContent = strstr($currentHtAccess , '#### END EXPIRES CACHING FOR GOOGLE BY PRESTACHAMPS ####');
        if($validateContent === false){
            $out = \Tools::file_get_contents(_PS_ROOT_DIR_ . '/.htaccess').$data;
            $regex = '#' . preg_quote('#### EXPIRES CACHING FOR GOOGLE BY PRESTACHAMPS ####', '#')
            . '(.*?)'
            . preg_quote('# EXPIRES CACHING FOR GOOGLE BY PRESTACHAMPS #', '#')
            . '#'
            . 's';

            return file_put_contents(
                _PS_ROOT_DIR_ . '/.htaccess',
                PHP_EOL . preg_replace($regex, '', $out)
            );
        }else{
            return;
        }

    }

    public function removeCacheFromHtAccess()
    {
        $currentHtAccess = \Tools::file_get_contents(_PS_ROOT_DIR_ . '/.htaccess');

        $removeText = '## EXPIRES CACHING FOR GOOGLE BY PRESTACHAMPS ##
        <IfModule mod_expires.c>
        ExpiresActive On
        ExpiresByType image/jpg "access 1 year"
        ExpiresByType image/jpeg "access 1 year"
        ExpiresByType image/gif "access 1 year"
        ExpiresByType image/png "access 1 year"
        ExpiresByType image/webp "access 1 year"
        ExpiresByType text/css "access 1 year"
        ExpiresByType application/pdf "access 1 year"
        ExpiresByType text/javascript "access 1 year"
        ExpiresByType text/x-javascript "access 1 month"
        ExpiresByType application/javascript "access 1 year"
        ExpiresByType application/x-javascript "access 1 month"
        ExpiresByType image/icon "access 1 month"
        ExpiresByType application/ico "access 1 month"
        ExpiresByType application/x-ico "access 1 month"
        ExpiresByType image/x-icon "access 1 year"
        ExpiresByType application/x-shockwave-flash "access 1 month"
        # Compress compressible fonts
        AddOutputFilterByType DEFLATE application/x-font-ttf application/x-font-opentype image/svg+xml
        # Add a far future Expires header for fonts
        ExpiresByType application/vnd.ms-fontobject "access 1 year"
        ExpiresByType application/x-font-ttf "access 1 year"
        ExpiresByType application/x-font-opentype "access 1 year"
        ExpiresByType application/x-font-woff "access 1 year"
        ExpiresByType application/x-font-woff2 "access 1 year"
        ExpiresByType font/woff2  "access 1 year"
        ExpiresByType font/woff "access 1 year"
        ExpiresByType image/svg+xml "access 1 year"
        ExpiresByType font/otf "access 1 year"
        ExpiresByType image/vnd.microsoft.icon "access 1 year"
        </IfModule>
        ## END EXPIRES CACHING FOR GOOGLE BY PRESTACHAMPS ##';

        $outHtAccess = preg_replace('/## EXPIRES CACHING FOR GOOGLE BY PRESTACHAMPS ##[\s\S]+?#### END EXPIRES CACHING FOR GOOGLE BY PRESTACHAMPS ####/', '', $currentHtAccess);

        return file_put_contents(
            _PS_ROOT_DIR_ . '/.htaccess', 
            PHP_EOL . $outHtAccess
        );
        
    }
}
