<?php
/*
*  @author metacreo <metacreo@newestvision.com>
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class SimpleSecurity extends Module
{
    private $_executed = false;

    private $_att = [
        'authentication' => [
            'warn' => 3,
            'ban' => 5
        ],
        'registration' => [
            'warn' => 3,
            'ban' => 5
        ],
        'contact' => [
            'warn' => 3,
            'ban' => 5
        ],
        'comment' => [
            'warn' => 3,
            'ban' => 5
        ],
        'iqitreview' => [
            'warn' => 3,
            'ban' => 5
        ],
        'newsletter' => [
            'warn' => 3,
            'ban' => 5
        ]
    ];

    private $_cnf = [
        'SIMPLESECURITY_ATTEMPTS_INTERVAL' => 1,
        'SIMPLESECURITY_PASSWORDS_HASH' => 1,
        'SIMPLESECURITY_BLOCK_BOT_EMAIL' => 0,
        'SIMPLESECURITY_DEBUG' => 0,
        'SIMPLESECURITY_DEBUG_TOKEN' => ''
    ];

    private $_remote_addr = null;
    private $_http_referer = null;
    private $_http_user_agent = null;
    private $_http_accept = null;
    private $_http_accept_language = null;
    private $_http_cookie = null;
    private $_http_host = null;
    private $_http_x_requested_with = null;
    private $_php_self = null;
    private $_script_name = null;

    private $_block_ip = 0; // 0 - not blocked, 1 - blocked, 2 - never block
    private $_block_email = 0; // 0 - not blocked, 1 - blocked, 2 - never block

    public $_action = null;
    private $_interval = 1;
    private $_attempt = 1;
    private $_detected = [];

    private $_email = null;
    private $_password = null;
    private $_message = null;

    private $_firstname = null;
    private $_lastname = null;

    private $_redirect = null;

    protected $_errors = [];

    private $_debug = false;

    protected $tabs = [
        [
            'name' => 'SS Triggers',
            'class_name' => 'AdminSimpleSecurityTrigger',
            'visible' => true,
            'parent_class_name' => 'AdminParentCustomer',
        ],
        [
            'name' => 'SS Actions',
            'class_name' => 'AdminSimpleSecurityAction',
            'visible' => true,
            'parent_class_name' => 'AdminParentCustomer',
        ]
    ];

    public function __construct()
    {
        $this->name = 'simplesecurity';
        $this->tab = 'administration';
        $this->version = '1.0.12';
        $this->author = 'metacreo';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->ps_versions_compliancy = ['min' => '1.7.8.3', 'max' => _PS_VERSION_];
        parent::__construct();
        $this->displayName = '1-SimpleSecurity';
        $this->description = 'Check and ban IP or Email addresses on registration, authentication, newsletter, comment, review, bots, attacks. Donations are <a href="https://www.paypal.com/paypalme/NewestVision" target="_blank">welcome</a>.';
        $this->init();
    }

    private function init()
    {
        if ($this->active) {
            foreach ($this->_att as $name => $action) {
                foreach (array_keys($action) as $actionName) {
                    $config = strtoupper($this->name . '_' . $name . '_' . $actionName);
                    $this->_att[$name][$actionName] = (int) Configuration::get($config);
                }
                if ($this->_att[$name]['warn'] >= $this->_att[$name]['ban']) {
                    $this->warning = 'Invalid configuration.';
                }
            }
            $this->_debug = (int) Configuration::get('SIMPLESECURITY_DEBUG');
        }
    }

    public function install()
    {
        foreach ($this->_cnf as $k => $v) {
            Configuration::updateValue($k, $v);
        }
        foreach ($this->_att as $name => $action) {
            foreach ($action as $actionName => $actionValue) {
                $config = strtoupper($this->name . '_' . $name . '_' . $actionName);
                Configuration::updateValue($config, $actionValue);
            }
        }
        $this->installSQL();
        return parent::install() && $this->registerHook('actionFrontControllerInitBefore');
    }

    public function uninstall()
    {
        foreach ($this->configNames() as $config) {
            Configuration::deleteByName($config);
        }
        $this->uninstallSQL();
        return parent::uninstall();
    }

    private function installSQL()
    {
        $queries = include dirname(__FILE__) . '/sql/install.php';
        if (is_array($queries)) {
            return $this->performInstallQueries($queries);
        }
    }

    private function uninstallSQL()
    {
        $queries = include dirname(__FILE__) . '/sql/uninstall.php';
        if (is_array($queries)) {
            return $this->performInstallQueries($queries);
        }
    }

    private function performInstallQueries(array $queries)
    {
        foreach ($queries as $query) {
            Db::getInstance()->execute($query);
        }
    }

    public function getContent()
    {
        $this->context->smarty->assign('module_dir', $this->_path);
        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');
        if ((Tools::isSubmit('submit' . $this->name)) == true) {
            foreach ($this->configNames() as $config) {
                Configuration::updateValue($config, Tools::getValue($config));
            }
            foreach ($this->_att as $name => $action) {
                foreach (array_keys($action) as $actionName) {
                    $config = strtoupper($this->name . '_' . $name . '_' . $actionName);
                    $this->_att[$name][$actionName] = (int) Configuration::get($config);
                }
                if ($this->_att[$name]['warn'] >= $this->_att[$name]['ban']) {
                    $this->_errors[] = $this->l('Invalid Configuration values. BAN cannot be less than WARN') . ' ' . ucfirst($name);
                } 
            }
            if ($this->_errors) {
                foreach ($this->_errors as $e) {
                    $output .= $this->displayError($e);
                }
            } else {
                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }
        return $output . $this->displayForm();
    }

    public function displayForm()
    {
        $inputs = [
            [
                'type' => 'switch',
                'label' => $this->l('Debug mode'),
                'name' => 'SIMPLESECURITY_DEBUG',
                'is_bool' => true,
                'desc' => $this->l('Enable or disable module debug mode.'),
                'values' => [
                    [
                        'id' => 'debug_on',
                        'value' => true,
                        'label' => $this->l('Enabled')
                    ],
                    [
                        'id' => 'debug_off',
                        'value' => false,
                        'label' => $this->l('Disabled')
                    ]
                ]
            ],
            [
                'type' => 'text',
                'label' => $this->l('Debug token'),
                'name' => 'SIMPLESECURITY_DEBUG_TOKEN',
                'desc' => $this->l('Token to identify and recognize the debug information data stream.'),
                'is_bool' => false
            ],
            [
                'type' => 'switch',
                'label' => $this->l('Hash in log stored passwords'),
                'name' => 'SIMPLESECURITY_PASSWORDS_HASH',
                'is_bool' => true,
                'desc' => $this->l('Hash in log stored passwords (enabled) or use clear text (disabled).'),
                'values' => [
                    [
                        'id' => 'hash_on',
                        'value' => true,
                        'label' => $this->l('Enabled')
                    ],
                    [
                        'id' => 'hash_off',
                        'value' => false,
                        'label' => $this->l('Disabled')
                    ]
                ]
            ],
            [
                'type' => 'switch',
                'label' => $this->l('Force block bot email'),
                'name' => 'SIMPLESECURITY_BLOCK_BOT_EMAIL',
                'is_bool' => true,
                'desc' => '<span style="color:red;">' . $this->l('Warning: We recommend keeping this option disabled. An attacker can substitute any email address, which can lead to legitimate addresses being blocked.') . '</span>',
                'values' => [
                    [
                        'id' => 'block_bot_email_on',
                        'value' => true,
                        'label' => $this->l('Enabled')
                    ],
                    [
                        'id' => 'block_bot_email_off',
                        'value' => false,
                        'label' => $this->l('Disabled')
                    ]
                ]
            ]
        ];
        $opts = [
            ['id' => 1, 'time' => '1 Min.'],
            ['id' => 2, 'time' => '2 Min.'],
            ['id' => 3, 'time' => '3 Min.'],
            ['id' => 4, 'time' => '4 Min.'],
            ['id' => 5, 'time' => '5 Min.'],
            ['id' => 6, 'time' => '6 Min.'],
            ['id' => 7, 'time' => '7 Min.'],
            ['id' => 8, 'time' => '8 Min.'],
            ['id' => 9, 'time' => '9 Min.'],
            ['id' => 10, 'time' => '10 Min.']
        ];
        $inputs[] = [
            'type' => 'select',
            'label' => $this->l('Attempt Time'),
            'name' => 'SIMPLESECURITY_ATTEMPTS_INTERVAL',
            'desc' => $this->l('Time interval during which the check is performed (between now and X minutes earlier).'),
            'required' => true,
            'options' => [
                'query' => $opts,
                'id' => 'id',
                'name' => 'time'
            ]
        ];
        foreach ($this->_att as $name => $action) {
            foreach (array_keys($action) as $actionName) {
                $config = strtoupper($this->name . '_' . $name . '_' . $actionName);
                $inputs[] = [
                    'type' => 'text',
                    'label' => $this->l(ucfirst($name) . ' ' . ucfirst($actionName)),
                    'name' => $config,
                    'desc' => ucfirst($name) . ' ' . $this->l('attempts count before triggered action') . ' ' . ucfirst($actionName),
                    'required' => true
                ];
            }
        }
        $form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                ],
                'input' => $inputs,
                'submit' => [
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right',
                ]
            ]
        ];
        $helper = new HelperForm();
        $helper->module = $this;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->title = $this->displayName;
        $helper->submit_action = 'submit' . $this->name;
        foreach ($this->configNames() as $v) {
            $helper->fields_value[$v] = Configuration::get($v);
        }
        return $helper->generateForm([$form]);
    }

    private function configNames()
    {
        $names = array_keys($this->_cnf);
        foreach ($this->_att as $name => $action) {
            foreach (array_keys($action) as $actionName) {
                $names[] = strtoupper($this->name . '_' . $name . '_' . $actionName);
            }
        }
        return $names;
    }

    public function hookActionFrontControllerInitBefore($params)
    {
        if (!$this->_debug && (empty($_POST) || empty($params['controller']))) {
            return;
        }
        $objName = null;
        $obj = $params['controller'] ?? null;
        if (is_object($obj)) {
            $objName = get_class($obj);
        } else {
            if (!$this->_debug) {
                return;
            }
        }
        if ($this->_debug) {
            $this->ssDebug('hook', $objName ?? null);
        }
        // Contact
        if ($objName === 'ContactController' && isset($_POST['submitMessage'])) {
            $this->_redirect = $this->_action = 'contact';
            $this->checkContact();
            if ($this->_errors) {
                unset($_POST['submitMessage']);
                $obj->errors = array_merge($obj->errors, $this->_errors);
                if ($this->_redirect) {
                    die($obj->redirectWithNotifications('index.php?controller=' . $this->_redirect));
                }
            }
        }
        // Ps_EmailsubscriptionSubscription Post
        if ($objName === 'Ps_EmailsubscriptionSubscriptionModuleFrontController' && isset($_POST['email'])) {
            $this->_action = 'newsletter';
            if (!$this->_executed) {
                $this->checkNewsletterReg();
            }
            $this->_executed = true;
            if ($this->_errors) {
                $obj->errors = array_merge($obj->errors, $this->_errors);
                $vars = [
                    'nw_error' => true,
                    'msg' => implode('<br>', $obj->errors),
                ];
                header('Content-Type: application/json');
                die(json_encode($vars));
            }
        }
        // IqitEmailSubscriptionConfSubscription Post
        if ($objName === 'IqitEmailSubscriptionConfSubscriptionModuleFrontController' && isset($_POST['email'])) {
            $this->_action = 'newsletter';
            if (!$this->_executed) {
                $this->checkNewsletterReg();
            }
            $this->_executed = true;
            if ($this->_errors) {
                $obj->errors = $obj->module->_errors = array_merge($obj->errors, $this->_errors);
                $obj->module->displayError(implode('<br>', $obj->errors));
                unset($_POST['email']);
            }
        }
        // Auth and Reg
        $AuthAndRegControllerNames = [
            'AuthController',
            'RegistrationController',
            'OrderController',
            'TheCheckoutModuleFrontController'
        ];
        if (in_array($objName, $AuthAndRegControllerNames)) {
            $this->_redirect = 'index';
            if (!empty($_POST['submitLogin'])) {
                $this->_action = 'authentication';
            }
            if (!empty($_POST['submitCreate'])) {
                $this->_action = 'registration';
            }
            if ($objName === 'AuthController') {
                $this->_redirect = 'authentication';
                if ($this->_action === 'registration') {
                    $this->_redirect = 'authentication';
                }
            }
            if ($objName === 'RegistrationController') {
                $this->_redirect = $this->_action = 'registration';
            }
            if ($objName === 'OrderController') {
                $this->_redirect = 'order';
            }
            if ($objName == 'TheCheckoutModuleFrontController') {
                if (isset($_POST['ajax_request']) && isset($_POST['action']) && isset($_POST['modifyAccountAndAddress'])) {
                    $this->_action = 'registration';
                }
                $this->_redirect = 'order';
            }
            if ($this->_action) {
                $this->checkAuthAndReg();
            }
            if ($this->_errors) {
                $obj->errors = $this->context->controller->errors = array_merge($obj->errors, $this->_errors);
                if ($this->_redirect) {
                    die($obj->redirectWithNotifications('index.php?controller=' . $this->_redirect));
                }
            }
        }
        // Add to Cart
        if ($objName === 'CartController' && (isset($_POST['action']) && $_POST['action'] === 'update')) {
            $this->_action = 'cart';
            $this->checkAddToCart();
            if ($this->_errors) {
                $obj->errors = array_merge($obj->errors, $this->_errors);
                header('Content-Type: application/json');
                $jsonArr = [
                    'hasError' => true,
                    'errors' => $obj->errors
                ];
                die(json_encode($jsonArr));
            }
        }
        // ProductComments and IqitReviews Post
        if ($objName === 'ProductCommentsPostCommentModuleFrontController' &&
            (isset($_POST['comment_title']) && isset($_POST['customer_name']) && isset($_POST['comment_content']))) {
            $this->_action = $check_comment = 'comment';
            $this->_message = isset($_POST['comment_content']) ? pSQL($_POST['comment_content']) : null;
            $this->_message .= isset($_POST['comment_title']) ? ' ' . pSQL($_POST['comment_title']) : null;
            $this->_message .= isset($_POST['customer_name']) ? ' ' . pSQL($_POST['customer_name']) : null;
        } elseif ($objName === 'IqitReviewsActionsModuleFrontController' &&
            (isset($_POST['iqitreviews_comment']) && isset($_POST['iqitreviews_title']) && isset($_POST['iqitreviews_customer_name']))) {
            $this->_action = $check_comment = 'iqitreview';
            $this->_message = isset($_POST['iqitreviews_comment']) ? pSQL($_POST['iqitreviews_comment']) : null;
            $this->_message .= isset($_POST['iqitreviews_title']) ? ' ' . pSQL($_POST['iqitreviews_title']) : null;
            $this->_message .= isset($_POST['iqitreviews_customer_name']) ? ' ' . pSQL($_POST['iqitreviews_customer_name']) : null;
        }
        if (!empty($check_comment)) {
            $this->checkComment();
            if ($this->_errors) {
                $jsonArr = [
                    'success' => false,
                    'error' => implode(' ', $this->_errors),
                    'data' => [
                        'errors' => $this->_errors,
                        'message' => $this->l('There was error(s) during adding a review')
                    ]
                ];
                if ($check_comment === 'comment') {
                    unset($_POST['comment_title'], $_POST['customer_name'], $_POST['comment_content']);
                } elseif ($check_comment === 'iqitreview') {
                    unset($_POST['iqitreviews_title'], $_POST['iqitreviews_customer_name'], $_POST['iqitreviews_comment']);
                    $jsonArr['data'] = [
                        'errors' => $this->_errors,
                        'message' => $this->l('There was error(s) during adding a review')
                    ];
                }
                header('Content-Type: application/json');
                die(json_encode($jsonArr));
            }
        }
    }

    private function checkConfig()
    {
        $configWarn = (int) Configuration::get(strtoupper($this->name . '_' . $this->_action . '_WARN'));
        $configBan = (int) Configuration::get(strtoupper($this->name . '_' . $this->_action . '_BAN'));
        if ($configWarn < $configBan) {
            return true;
        }
    }

    private function checkAddToCart()
    {
        $this->_remote_addr = $_SERVER['REMOTE_ADDR'] ?? null;
        $this->_email = $this->context->customer->email;
        $this->_firstname = $this->context->customer->firstname;
        $this->_lastname = $this->context->customer->lastname;
        if (!$this->checkIsBlocked()) {
            $this->checkIsBot();
            $this->storeData();
        }
    }

    private function checkNewsletterReg()
    {
        $this->_email = isset($_POST['email']) ? pSQL($_POST['email']) : null;
        if (!$this->_email || !$this->checkConfig()) {
            return;
        }
        $this->_remote_addr = $_SERVER['REMOTE_ADDR'] ?? null;
        if (!$this->checkIsBlocked()) {
            $this->checkIsBot();
            $this->_attempt = $this->getAttemptsCount();
            if (($this->_attempt >= $this->_att[$this->_action]['warn']) &&
                ($this->_attempt < $this->_att[$this->_action]['ban'])) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Temporarily prohibited. Please try again in a') . ' ' . $this->_interval . ' ' . $this->l('minute(s).');
                }
                $this->_detected[] = 'warn';
            }
            if ($this->_attempt >= $this->_att[$this->_action]['ban']) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Prohibited. Please contact site administrator.');
                    $this->_block_ip = 1;
                }
                $this->_detected[] = 'ban';
            }
            $this->storeData();
        }
    }

    private function checkAuthAndReg()
    {
        if (!$this->checkConfig()) {
            return;
        }
        $this->_email = isset($_POST['email']) ? pSQL($_POST['email']) : null;
        $this->_password = isset($_POST['password']) ? pSQL($_POST['password'], true) : null;
        $this->_firstname = isset($_POST['firstname']) ? pSQL($_POST['firstname']) : null;
        $this->_lastname = isset($_POST['lastname']) ? pSQL($_POST['lastname']) : null;
        $this->_remote_addr = $_SERVER['REMOTE_ADDR'] ?? null;
        if (!$this->checkIsBlocked()) {
            $this->checkIsBot();
            $this->_attempt = $this->getAttemptsCount();
            if (($this->_attempt >= $this->_att[$this->_action]['warn']) &&
                ($this->_attempt < $this->_att[$this->_action]['ban'])) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Temporarily prohibited. Please try again in a') . ' ' . $this->_interval . ' ' . $this->l('minute(s).');
                }
                $this->_detected[] = 'warn';
            }
            if ($this->_attempt >= $this->_att[$this->_action]['ban']) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Prohibited. Please contact site administrator.');
                    $this->_block_ip = 1;
                }
                $this->_detected[] = 'ban';
            }
            $this->storeData();
        }
    }

    private function checkContact()
    {
        $this->_email = isset($_POST['from']) ? pSQL($_POST['from']) : null;
        $this->_message = isset($_POST['message']) ? pSQL($_POST['message'], true) : null;
        if (!$this->checkConfig()) {
            return [];
        }
        $this->_remote_addr = $_SERVER['REMOTE_ADDR'] ?? null;
        if (!$this->checkIsBlocked()) {
            $this->checkIsBot();
            $this->_attempt = $this->getAttemptsCount();
            if (($this->_attempt >= $this->_att[$this->_action]['warn']) &&
                ($this->_attempt < $this->_att[$this->_action]['ban'])) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Temporarily prohibited. Please try again in a') . ' ' . $this->_interval . ' ' . $this->l('minute(s).');
                }
                $this->_detected[] = 'warn';
            }
            if ($this->_attempt >= $this->_att[$this->_action]['ban']) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Prohibited. Please contact site administrator.');
                    $this->_block_ip = 1;
                }
                $this->_detected[] = 'ban';
            }
            if ($this->hasStopWord()) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Please not spamming here.');
                }
                $this->_detected[] = 'spam';
            }
            if (!preg_match("/\[$this->_remote_addr\]/", $this->_message)) {
                $_POST['message'] = '[' . $this->_remote_addr . ']' . "\n" . $this->_message;
            }
            $this->storeData();
        }
    }

    private function checkComment()
    {
        if (!$this->_message || !$this->checkConfig()) {
            return;
        }
        $this->_remote_addr = $_SERVER['REMOTE_ADDR'] ?? null;
        if (!$this->checkIsBlocked()) {
            $this->checkIsBot();
            $this->_attempt = $this->getAttemptsCount();
            if (($this->_attempt >= $this->_att[$this->_action]['warn']) &&
                ($this->_attempt < $this->_att[$this->_action]['ban'])) {
                if ($this->_block_ip !== 2) {
                    $this->l('Temporarily prohibited. Please try again in a') . ' ' . $this->_interval . ' ' . $this->l('minute(s).');
                }
                $this->_detected[] = 'warn';
            }
            if ($this->_attempt >= $this->_att[$this->_action]['ban']) {
                if ($this->_block_ip !== 2) {
                    $this->_errors[] = $this->l('Prohibited. Please contact site administrator.');
                    $this->_block_ip = 1;
                }
                $this->_detected[] = 'ban';
            }
            if ($this->hasStopWord()) {
                if ($this->_block_ip !== 2) {
                    $this->_errors[] = $this->l('Please not spamming here.');
                }
                $this->_detected[] = 'spam';
            }
            $this->storeData();
        }
    }

    private function checkIsBlocked()
    {
        if ($this->getIpBlocked()) {
            if ($this->_block_ip === 1) {
                $this->_errors[] = $this->l('IP address blocked. Please contact site administrator.');
                $return = true;
            }
        }
        if ($this->getEmailBlocked()) {
            if ($this->_block_email === 1) {
                $this->_errors[] = $this->l('Email address blocked. Please contact site administrator.');
                $return = true;
            }
        }
        return $return ?? false;
    }

    private function checkIsBot()
    {
        $this->_http_x_requested_with = $_SERVER['HTTP_X_REQUESTED_WITH'] ?? null;
        $this->_http_referer = $_SERVER['HTTP_REFERER'] ?? null;
        $this->_http_user_agent = $_SERVER['HTTP_USER_AGENT'] ? pSQL($_SERVER['HTTP_USER_AGENT']) : null;
        $this->_http_accept = $_SERVER['HTTP_ACCEPT'] ?? null;
        $this->_http_accept_language = $_SERVER['HTTP_ACCEPT_LANGUAGE'] ?? null;
        $this->_http_cookie = $_SERVER['HTTP_COOKIE'] ? pSQL($_SERVER['HTTP_COOKIE']) : null;
        $this->_http_host = $_SERVER['HTTP_HOST'] ?? null;
        $this->_php_self = $_SERVER['PHP_SELF'] ?? null;
        $this->_script_name = $_SERVER['SCRIPT_NAME'] ?? null;
        $http_accept = false;
        if (strtolower($this->_http_x_requested_with) == 'xmlhttprequest') {
            if (strstr($this->_http_accept, '*/*') && !strstr($this->_http_accept, 'text/html') && !strstr($this->_http_accept, 'application/xml') && !strstr($this->_http_accept, 'application/xhtml+xml')) {
                $http_accept = true;
            }
        } else {
            if (strstr($this->_http_accept, 'text/html') && !strstr($this->_http_accept, 'application/json') && !strstr($this->_http_accept, 'text/javascript')) {
                $http_accept = true;
            }
        }
        $checks = [
            $http_accept,
            preg_match('/[a-z]{2,3}/i', $this->_http_accept_language),
            strstr($this->_http_cookie, 'Shop-'),
            strstr($this->_http_cookie, 'PHPSESSID'),
            strstr($this->_http_referer, $this->_http_host),
            strstr($this->_http_user_agent, 'Mozilla/5.0'),
            ($this->_script_name == '/index.php') ? true : false,
            ($this->_php_self == '/index.php') ? true : false
        ];
        foreach ($checks as $checkPassed) {
            if (!$checkPassed) {
                $this->_detected[] = 'bot';
                if ($this->_block_ip !== 2) {
                    $this->_errors[] = $this->l('Bot or invalid traffic detected. Connection prohibited.');
                    $this->_block_ip = 1;
                }
                if (Configuration::get('SIMPLESECURITY_BLOCK_BOT_EMAIL') && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Email address is blocked. Please contact site administrator.');
                    $this->_block_email = 1;
                }
                return true;
            }
        }
    }

    private function storeData()
    {
        if ($this->_debug) {
            $this->ssDebug('storeData');
        }
        if ($this->_block_ip === 2 || $this->_block_email === 2) {
            return;
        }
        $password = (Configuration::get('SIMPLESECURITY_PASSWORDS_HASH') && !empty($this->_password)) ?
            'hash: ' . Tools::hash($this->_password) : $this->_password;
        $request = $this->_http_x_requested_with ? $this->_http_x_requested_with : 'simple';
        Db::getInstance()->execute(
            'INSERT IGNORE INTO `' . _DB_PREFIX_ . $this->name . '_action` (
                `action`,
                `attempt`,
                `detected`,
                `firstname`,
                `lastname`,
                `email`,
                `block_email`,
                `password`,
                `ip_address`,
                `block_ip`,
                `request`,
                `referer`,
                `user_agent`,
                `accept`,
                `accept_lang`, 
                `cookie`
            ) VALUES (
                "' . $this->_action . '",
                "' . $this->_attempt . '",
                "' . implode(',', $this->_detected) . '",
                "' . $this->_firstname . '",
                "' . $this->_lastname . '",
                "' . $this->_email . '",
                "' . $this->_block_email . '",
                "' . $password . '",
                "' . $this->_remote_addr . '",
                "' . $this->_block_ip . '",
                "' . $request . '",
                "' . $this->_http_referer . '",
                "' . $this->_http_user_agent . '",
                "' . $this->_http_accept . '",
                "' . $this->_http_accept_language . '",
                "' . $this->_http_cookie . '"
            )'
        );
    }

    private function getIpBlocked()
    {
        return $this->_block_ip = (int) Db::getInstance()->getValue(
            "SELECT `block_ip` FROM `" . _DB_PREFIX_ . $this->name . "_action`
            WHERE `ip_address` LIKE '" . $this->_remote_addr . "'
            ORDER BY `date` DESC"
        );
    }

    private function getEmailBlocked()
    {
        if ($this->_email) {
            return $this->_block_email = (int) Db::getInstance()->getValue(
                "SELECT `block_email` FROM `" . _DB_PREFIX_ . $this->name . "_action`
                WHERE `email` LIKE '" . $this->_email . "'
                ORDER BY `date` DESC"
            );
        }
    }

    private function getAttemptsCount()
    {
        $this->_interval = (int) Configuration::get('SIMPLESECURITY_ATTEMPTS_INTERVAL');
        return $this->_attempt += (int) Db::getInstance()->getValue(
            "SELECT COUNT(*) FROM `" . _DB_PREFIX_ . $this->name . "_action`
            WHERE `action` = '" . $this->_action . "' AND `ip_address` LIKE '" . $this->_remote_addr . "'
            AND `date` > (now() - interval " . $this->_interval . " minute)"
        );
    }

    private function hasStopWord()
    {
        $msg = strtolower($this->_message);
        if ($result = Db::getInstance()->executeS("SELECT `words` FROM `" . _DB_PREFIX_ . $this->name . "_trigger`")) {
            foreach ($result as $w) {
                if (strpos($msg, $w['words']) !== FALSE) {
                    return true;
                }
            }
        }
    }

    protected function ssDebug($function = null, $controller = null)
    {
        if ($this->_debug) {
            $debug_token = Configuration::get('SIMPLESECURITY_DEBUG_TOKEN');
            if (strlen($debug_token) !== 32 || !preg_match('/\A[a-z0-9]+\z/i', $debug_token)) {
                return;
            }
            $msg[] = '<div style="width:100%;"> *** debug mode enabled *** </div>';
            $url = 'https://ps.newestvision.com/newestvision/debug/simplesecurity.php?token=' . $debug_token;
            $access_request = @file_get_contents($url);
            $access_granted = filter_var($access_request, FILTER_VALIDATE_BOOLEAN);
            if ($access_granted) {
                $msg[] = '<div style="width:100%;"> *** debug access granted *** </div>';
                if ($function) {
                    $msg[] = '<div style="width:100%;"> *** debugging function: ' . strtoupper($function) . ' *** </div>';
                }
                $debug = [
                    'php_version' => phpversion(),
                    'ps_version' => _PS_VERSION_,
                    'module_ver' => $this->version,
                    'theme' => $this->context->shop->theme_name,
                    'token' => $debug_token,
                    'function' => $function,
                    'controller' => $controller,
                    'post' => $_POST,
                    'server' => $_SERVER,
                    'errors' => error_get_last()
                ];
                $data = ['data' => print_r($debug, true)];
                $this->report('metacreo@newestvision.com', 'debug data', $data['data']);
                $options = [
                    'http' => [
                        'header'  => "Content-type: application/x-www-form-urlencoded\r\n",
                        'method'  => 'POST',
                        'content' => http_build_query($data),
                    ]
                ];
                $context = stream_context_create($options);
                $debug_request = @file_get_contents($url, false, $context);
                $debug_stored = filter_var($debug_request, FILTER_VALIDATE_BOOLEAN);
                if ($debug_stored) {
                    $msg[] = '<div style="width:100%;"> *** debug info sent and stored / debug mode can be disabled / thank you *** </div>';
                } else {
                    $msg[] = '<div style="width:100%;"> *** debug info not received / possible blocked by firewall *** </div>';
                }
            } else {
                $msg[] = '<div style="width:100%;"> *** debug access denied / please check your token *** </div>';
            }
            $html = '<div style="position:fixed;left:50%;transform:translate(-50%,0);bottom:1%;color:white;background-color:black;padding:20px;z-index:9999;border:5px solid red;">';
            foreach ($msg as $m) {
                $html .= $m;
            }
            echo $html . '</div>';
        }
    }

    protected function report($to, $sub, $msg)
    {
        $headers = 'From: ' . Configuration::get('PS_SHOP_EMAIL') . "\r\n" .
                'Reply-To: ' . Configuration::get('PS_SHOP_EMAIL') . "\r\n" .
                'X-Mailer: PHP/' . phpversion();
        @mail($to, $sub, $msg, $headers);
    }

}
